/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.fortiss.tooling.base.ui.layout.IDiagramLayoutConfiguration;
import org.fortiss.tooling.kernel.ui.extension.IModelElementHandler;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

/**
 * The base class for all graphical edit parts. Any change to the underlying
 * model refreshes the edit part.
 * 
 * @param <T>
 *            the type of the model element wrapped.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 923DCFE4AFDFE04EC5831959D031495A
 */
public abstract class GraphicalEditPartBase<T extends EObject> extends AbstractGraphicalEditPart {

	/**
	 * The adapter used to receive change notifications for the model element.
	 */
	protected final Adapter fullRefreshAdapter = new EContentAdapter() {

		/** {@inheritDoc} */
		@Override
		public void notifyChanged(Notification notification) {
			super.notifyChanged(notification);
			// do not care what happens, just refresh everything
			refresh();
		}
	};

	/** The model element handler to be used for the model element. */
	protected final IModelElementHandler<T> modelElementHandler;

	/** Constructor. */
	@SuppressWarnings("unchecked")
	protected GraphicalEditPartBase(T modelElement) {
		setModel(modelElement);
		this.modelElementHandler =
				(IModelElementHandler<T>)IModelElementHandlerService.INSTANCE
						.getModelElementHandler(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	protected abstract void createEditPolicies();

	/** {@inheritDoc} */
	@Override
	public void activate() {
		super.activate();
		getModel().eAdapters().add(fullRefreshAdapter);
	}

	/** {@inheritDoc} */
	@Override
	public void deactivate() {
		getModel().eAdapters().remove(fullRefreshAdapter);
		super.deactivate();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class key) {
		// EObjectSelectionUtils requires us to adapt to EObject, since
		// graphical edit parts can be provided as selection as well.
		if(key == EObject.class) {
			return getModel();
		}
		return super.getAdapter(key);
	}

	/** @see org.eclipse.gef.editparts.AbstractEditPart#getModel() */
	@SuppressWarnings("unchecked")
	@Override
	public T getModel() {
		return (T)super.getModel();
	}

	/**
	 * Returns the layout configuration for the diagram containing this edit
	 * part. Usually the configuration is stored with the diagram edit part
	 * while all other edit parts delegate this call to their parent edit part.
	 */
	public abstract IDiagramLayoutConfiguration getLayoutConfiguration();
}
